@@ -1,7 +1,7 @@
 // SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
 // -*- Mode: C++ -*-
 //
-// Copyright (C) 2016-2020 Red Hat, Inc.
+// Copyright (C) 2016-2022 Red Hat, Inc.
 
 /// @file
 ///
@@ -670,7 +670,7 @@ struct corpus::priv
   environment*					env;
   corpus_group*				group;
   corpus::exported_decls_builder_sptr		exported_decls_builder;
-  origin					origin_;
+  corpus::origin				origin_;
   vector<string>				regex_patterns_fns_to_suppress;
   vector<string>				regex_patterns_vars_to_suppress;
   vector<string>				regex_patterns_fns_to_keep;
