@@ -1,7 +1,7 @@
 // SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
 // -*- Mode: C++ -*-
 //
-// Copyright (C) 2020 Google, Inc.
+// Copyright (C) 2020-2022 Google, Inc.
 
 /// @file
 ///
@@ -49,6 +49,9 @@ find_section(Elf*		elf_handle,
 	     const std::string& name,
 	     Elf64_Word		section_type);
 
+Elf_Scn*
+find_section_by_name(Elf* elf_handle, const std::string& name);
+
 Elf_Scn*
 find_section(Elf* elf_handle, Elf64_Word section_type);
 
@@ -112,6 +115,10 @@ find_ksymtab_strings_section(Elf *elf_handle);
 Elf_Scn*
 find_relocation_section(Elf* elf_handle, Elf_Scn* target_section);
 
+Elf_Scn*
+find_strtab_for_symtab_section(Elf*	elf_handle,
+                               Elf_Scn*	symtab_section);
+
 //
 // Helpers for symbol versioning
 //
@@ -140,9 +147,15 @@ get_version_for_symbol(Elf*			elf_handle,
 bool
 architecture_is_ppc64(Elf* elf_handle);
 
+bool
+architecture_is_ppc32(Elf* elf_handle);
+
 bool
 architecture_is_arm32(Elf* elf_handle);
 
+bool
+architecture_is_arm64(Elf* elf_handle);
+
 bool
 architecture_is_big_endian(Elf* elf_handle);
 
